!========================================
! Copyright  2001 by Carl Barnes
!========================================

  MEMBER

 OMIT('*endomit*',_C55_)
_ABCDllMode_    EQUATE(0)
_ABCLinkMode_   EQUATE(1)
 *endomit*

  INCLUDE('CBSHAuto.inc'),ONCE
  INCLUDE('LoadLib.inc'),ONCE

 compile('*** C6 ***',_c60_)
  INCLUDE('CWSYNCHC.INC'),ONCE
  INCLUDE('CBCSGlobal.inc'),ONCE    !C6
CSGlobal    CBCSGlobalClass         !The GLobal Critical Section that anybody can use
  !*** C6 ***

S_OK           EQUATE(0)
S_FALSE        EQUATE(1)

DWORD          EQUATE(LONG)
HRESULT        EQUATE(LONG)
HWND           EQUATE(UNSIGNED)

! Severity of error, use ABC Codes here
Level:Benign   EQUATE(0)
Level:User     EQUATE(1)

  MAP
      MODULE('Windows')
CoInitialize    PROCEDURE (<*LONG>),HRESULT,RAW,PROC,PASCAL
CoUninitialize  PROCEDURE (),RAW,PASCAL
      END

      MODULE('shlwapi.dll')    !Loaded by LoadDLLs class below, do not link shlwapi.lib
SHAutoComplete  PROCEDURE(HWND hwndEdit,DWORD dwFlags),HRESULT,PASCAL,PROC,DLL(_fp_)
      END

  END

! Function pointer variables to receive the address of the api function.
! You must assign a valid address BEFORE you attempt to call the function.
fpSHAutoComplete    LONG,NAME('SHAutoComplete')

!=====================================================================================
!--I want the DLL loaded just once (when required) and not unloaded. This class should do that
LoadDLLs        class
Init                PROCEDURE(),short            !0=No 1=Yes -1=Failed
Destruct            PROCEDURE()
shlwapi             &LoadLibClass,PROTECTED      !Larry Sand LoadLib
InitStatus          short,private                !0=No 1=Yes -1=Failed
                end
LoadDLLs.Init   PROCEDURE()
RetVal  LONG,AUTO
 compile('*** C6 ***',_c60_)
CSProc  CriticalProcedure       !C6
 *** C6 ***
    CODE
    IF SELF.InitStatus THEN           !Has init been tried
       RETURN CHOOSE(SELF.InitStatus>0,Level:Benign,Level:User)
    END

    !In C6 this code should have a Critical Section Protecting it
    !Although unlikely, it is possible two threads could both be in the init code at the same time
    !I don't like the idea of the expensive of declaring a critical section here to protect this code
    !So I made a GlobalCS class that should only have a single CS for the entire EXE

 compile('*** C6 ***',_c60_)
    CSProc.Init(CSGlobal.GetIFace())        !Lock the Global Critical Section using Crit Proc so we know it gets unlocked
    !During the time it took to get the CS locked somebody else could have initted us
    !So check again and return that result
    !I did not want to put the CSLock above
?   ASSERT(ADDRESS(SELF.InitStatus % 4)=0)          !To insure the Above IF happens in 1 instruction it needs to be DWord aligned
    IF SELF.InitStatus THEN           !Has init been tried
       RETURN CHOOSE(SELF.InitStatus>0,Level:Benign,Level:User)
    END
 !*** C6 ***

    SELF.InitStatus = -1                    !Assume we failed
    RetVal = 1
    !test RETURN 1 !Simulate failed DLL load
    SELF.shlwapi &= NEW LoadLibClass
    IF NOT SELF.shlwapi &= NULL
       RetVal = SELF.shlwapi.LlcLoadLibrary('shlwapi.dll', Method:GetModuleHandle)
       IF SELF.shlwapi.LibraryLoaded()
          fpSHAutoComplete = SELF.shlwapi.LlcGetProcAddress('SHAutoComplete')
          IF fpSHAutoComplete THEN          !Ever everything good
             SELF.InitStatus = 1
             RetVal=0
          END
       ELSE
          RetVal=1
          fpSHAutoComplete = 0
       END
    END
    RETURN RetVal

LoadDLLs.Destruct   PROCEDURE()
    CODE
    IF NOT SELF.shlwapi &= NULL
         DISPOSE(SELF.shlwapi)
    END
    SELF.InitStatus = 0
    RETURN
!=====================================================================================

!===================================================================
! Construct()
!===================================================================
SHAutoCompleteClass.Construct   PROCEDURE()
hr  HResult
  CODE
  hr=CoInitialize()
  CASE hr
  OF   S_Ok          !Com did init
  OROF S_False       !Com was previously inited, that's ok
       SELF.CoInitialized=True
  ELSE               !CoInit Failed
       SELF.CoInitialized=False
       SELF.LastHResult = hr
  END
  RETURN

!===================================================================
! Destruct()
!===================================================================
SHAutoCompleteClass.Destruct   PROCEDURE() 
  CODE
  IF SELF.CoInitialized
     CoUnInitialize()
  END
  RETURN

!Initing a  LoadLibClass that is part of this class will not work if this class
!is implemented by multiple threads
!===================================================================
! Init() with Library Loaded when Class is called
!===================================================================
SHAutoCompleteClass.Init   PROCEDURE()
RetVal  LONG,AUTO
  CODE
  IF SELF.InitStatus THEN           !Has init been tried
     RETURN CHOOSE(SELF.InitStatus>0,Level:Benign,Level:User)
  END

  SELF.InitStatus = -1                     !Assume we failed Init
  IF ~SELF.CoInitialized                   !Did CoInit fail in the Constructor?
      SELF.LastHResult = -2               
      RetVal = Level:User
  ELSE
      IF LoadDLLs.Init()                       !Is the DLL Loaded?
         SELF.LastHResult = -3                 !DLL Load Failed :(
         RetVal=Level:User

      ELSE
          !Everything init'd ok
          RetVal = 0
          SELF.InitStatus = True
      END
  END
  RETURN RetVal

!===================================================================
! Call these methods for a TEXT Control to Fill with File URL or All
!===================================================================

SHAutoCompleteClass.Input_File             PROCEDURE(LONG TextFEQ)!,LONG
    CODE
    RETURN SELF.Input_All(TextFEQ,SHACR_INPUT_File)
!===================================================================

SHAutoCompleteClass.Input_URL              PROCEDURE(LONG TextFEQ)!,LONG
    CODE
    RETURN SELF.Input_All(TextFEQ,SHACR_INPUT_URL)
!===================================================================

SHAutoCompleteClass.Input_All              PROCEDURE(LONG TextFEQ, LONG Flags=SHACR_INPUT_ALL)!,LONG
RetVal  LONG(0)
HR      HRESULT
    CODE
?   ASSERT(Flags = 0 OR BAND(Flags,0111b)>0,'No Fill Data Equates Selected')  !Assume if Mode was selected that Files or URLs were selected
    IF ~SELF.InitStatus
        RetVal=SELF.INIT()
    END
    IF RetVal=0             !Init did not fail
       HR=SHAutoComplete(TextFEQ{prop:handle},Flags)
       SELF.LastHResult = HR
       RetVal=CHOOSE(HR=S_Ok,Level:Benign,Level:User)
    END
    RETURN RetVal

!===================================================================
